<?php

/**
 * Simple component for handling SFTP functionalities
 * Require PECL ssh2 installed on your environment
 *
 * @author	Aruna Attanayake <aruna470@gmail.com>
 * @version 1.1
 */

class SftpComponent extends CApplicationComponent
{
	/**
     * @var string $host sftp host ip.
     */ 
	public $host = null;
	 
	/**
     * @var string $port sftp host port default 22.
     */
	public $port = '22';
	
	/**
     * @var string $username username of remote sftp account.
     */
	public $username = null;
	
	/**
     * @var string $username username of remote sftp account.
     */
	public $password = null;
		
	/**
     * @var string $connection sftp connection resouce.
     */
	private $connection = null;
	
	/**
     * @var string $sftp sftp connection resouce.
     */
	private $sftp = null;
	
	/**
	 * @param string $host
	 * @param string $username
	 * @param string $password
	 * @param string $port
	 */
	public function __construct($host = null, $username = null, $password = null, $port = '22')
	{
      	$this->host = $host;
      	$this->username = $username;
     	$this->password = $password;
      	$this->port = $port;
	}
	
	/**
	 * Initializes the component.
	 */
	public function init()
	{
		parent::init();
	}
	
	/**
	 * Connect to SFTP if it is currently not
	 * @return bool true when connection success
	 * @throws CException if connection fails
	 */
	public function connect() 
	{
		if (null == $this->connection)
		{
			if ($this->connection = @ssh2_connect($this->host, $this->port)) 
			{
				if (ssh2_auth_password($this->connection, $this->username, $this->password))
				{
					$this->sftp = @ssh2_sftp($this->connection); 
					return true;
				}
				else
				{
					throw new CException('Invalid username or password.');
				}
			} 
			else 
			{
				throw new CException('Host unreachable.');
			}
		}		
	}
	
	/**
	 * list directory contents
	 * @param string $directory Directory path
	 * @param bool $showHiddenFiles default false, if true list hidden files also
	 * @return array $files list of contents including directories
	 * @throws CException if directory open fails
	 */
	public function listFiles($directory = '', $showHiddenfiles = false)
	{
		$dir = "ssh2.sftp://{$this->sftp}/{$directory}";
		$dh = @opendir($dir);
		
		if ($dh)
		{		
			$files = array();
			
			while (false !== ($file = readdir($dh)))
			{
				if (!$showHiddenfiles && ('.' == $file || '..' == $file || '.' == $file[0]))
					continue;
				
				$files[] = $file;
			}
			
			closedir($dh);
			
			return $files;
		}
		else
		{
			throw new CException('Directory open fails.');
		}
	}

	/**
	 * Returns the current directory
	 * @return string Current directory path
	 */
	public function getCurrentDir()
	{
		$response = $this->execCmd('pwd');
		
		return trim($response);
	}

	/**
	 * Check for directory
	 * @param string $directory Directory path
	 * @return bool true if is a directory otherwise false
	 */
	public function isDir($directory)
	{
		$dir = "ssh2.sftp://{$this->sftp}/{$directory}";
		$dh = @opendir($dir);
		
		if ($dh)
		{
			closedir($dh);
			return true;
		}
		
		return false;
	}

	/**
	 * Put file to a sftp location
	 * @param string $localFile Local file path
	 * @param string $remoteFile Remote file path
	 * @param string $permission File permission default 777
	 * @return bool true if file transfer success
	 * @throws CException if file transfer fails
	 */
	public function sendFile($localFile, $remoteFile)
	{
		if (ssh2_scp_send($this->connection, $localFile, $remoteFile)) 
		{
			return true;
		} 
		else 
		{
			throw new CException('File send failed.');
		}
	}
	
	/**
	 * Get file from sftp location
	 * @param string $remoteFile Remote file path
	 * @param string $localFile Local file path
	 * @return bool true if file retreival success
	 * @throws CException if file transfer fails
	 */
	public function getFile($remoteFile, $localFile) 
	{
		if (@ssh2_scp_recv($this->connection, $remoteFile, $localFile)) 
		{
			return true;
		} 
		else 
		{
			throw new CException('File get failed.');
		}
	}
	
	/**
	 * Get file from sftp location.This is an alternate method when getFile is not working
	 * Experienced ssh2_scp_recv() not properly working sometimes.
	 * @param string $remoteFile Remote file path
	 * @param string $localFile Local file path
	 * @param integer $timeOut Tiemout value in seconds.Default 60 seconds
	 * @return bool true if file retreival success
	 * @throws CException if file retreival fails
	 */
	public function altGetFile($remoteFile, $localFile, $timeOut=60)
	{
		$context = stream_context_create(array(
		    	'http' => array(
		        'timeout' => $timeOut      
		    )
		));
		
		$file_content = @file_get_contents("ssh2.sftp://{$this->sftp}{$remoteFile}", false, $context);
		
		if ('' != $file_content)
		{
    		@file_put_contents($localFile, $file_content);
    		
    		return true;
		}
		else
		{
			throw new CException('File get failed.');
		}
	}
	 
	/**
	 * Retreive file attributes 
	 * @param string $file Remote file path
	 * @param string $attribute Required attribute (size, gid, uid, atime, mtime, mode)
	 * @return string Attribute value
	 */
	private function getFileStat($file, $attribute)
	{
		$statinfo = @ssh2_sftp_stat($this->sftp, $file);
		
		return $statinfo[$attribute];
	}
	
	/**
	 * Retreive file size 
	 * @param string $file Remote file path
	 * @return string File size
	 */
	function getSize($file)
	{
		return $this->getFileStat($file, 'size');
	}
	
	/**
	 * Retreive file modified datetime 
	 * @param string $file Remote file path
	 * @return string File modified timestamp
	 */
	function getMdtm($file)
	{
		return $this->getFileStat($file, 'mtime');
	}
	
	/**
	 * Retreive file created datetime 
	 * @param string $file Remote file path
	 * @return string File created timestamp
	 */
	function getAtime($file)
	{
		return $this->getFileStat($file, 'atime');
	}
	
	/**
	 * Create directory on sftp location
	 * @param string $directory Remote directory path
	 * @param string $permission Directory permission default 777
	 * @param bool $recursive Whether recursive directory creation
	 * @return bool true if directory creation success
	 * @throws CException if directory creation fails
	 */
	function createDirectory($directory, $permission = 777, $recursive = true)
	{
		if (@ssh2_sftp_mkdir($this->sftp, $directory, $permission, $recursive))
		{
			return true;
		}
		else
		{ 
			throw new CException('Directory creation failed.');
		}
	}
	
	/**
	 * Remove directory on sftp location
	 * @param string $directory Remote directory path
	 * @param bool $foreceRemove If true remove directory even it is not empty
	 * @return bool true if directory removal success
	 * @throws CException if directory removal fails
	 */
	function removeDirectory($directory, $foreceRemove=false)
	{
		if ($foreceRemove)
		{
			$this->execCmd("rm -rf {$directory}");
			
			return true;
		}
		else
		{
			if (@ssh2_sftp_rmdir($this->sftp, $directory))
			{
				return true;
			}
			else
			{ 
				throw new CException('Directory removal failed.');
			}
		}
	}
		
	/**
	 * Remove file on sftp location
	 * @param string $file Remote file path
	 * @return bool true if directory removal success
	 * @throws CException if directory removal fails
	 */
	function removeFile($file)
	{
		if (@ssh2_sftp_unlink($this->sftp, $file))
		{
			return true;
		}
		else
		{ 
			throw new CException('File removal failed.');
		}
	}
	
	/**
	 * Change directory ownership
	 * @param string $path Directory or file path
	 * @param string $user User
	 * @param string $group Group
	 * @param bool $recursive Change ownership to subcontens also
	 * @return bool true 
	 */
	function chown($path, $user, $group, $recursive=false)
	{
		if ($recursive)
		{
			$cmd = "chown -R {$user}:{$group} {$path}";
		}
		else
		{		
			$cmd = "chown {$user}:{$group} {$path}";		
		}
		
		$this->execCmd($cmd);

		return true;
	}
	
	/**
	 * Change directory permission
	 * @param string $path Directory or file path
	 * @param string $permission Permission
	 * @param bool $recursive Change permission to subcontens also
	 * @return bool true 
	 */
	function chmod($path, $permission, $recursive=false)
	{
		if ($recursive)
		{
			$cmd = "chmod -R {$permission} {$path}";
		}
		else
		{		
			$cmd = "chmod {$permission} {$path}";		
		}
		
		$this->execCmd($cmd);

		return true;
	}
	
	/**
	 * Execute command on remote shell
	 * @param string $cmd Command ex:pwd
	 * @return string $output Command output
	 */
	function execCmd($cmd)  
	{
		$stream = @ssh2_exec($this->connection, $cmd);
		stream_set_blocking($stream, true);
		
		$output = '';
		
		while ($get = fgets($stream))
		{
			$output .= $get;
		}
		
		return $output;
	}
}
?>